#Ifndef matrix_4d_t
#Define matrix_4d_t
#Include "vector_2d.bi"
#Include "vector_3d.bi"
#Include "vector_4d.bi"
#Include "matrix_4d.bi"
#Include "quaternion_4d.bi"

Type matrix_4d
	'm As Single Ptr Ptr
	As Single m(0 To 3,0 To 4)
	Declare Constructor()
	Declare Destructor()
	
	
	Declare Function InitIdentity() As matrix_4d
	Declare Function InitScreenSpaceTransform(halfwidth As single, halfheight As Single) As matrix_4d
	Declare Function InitTranslation(x As single, y As single, z As Single) As matrix_4d
	Declare Function InitRotation(x As single, y As single, z As Single) As matrix_4d
	Declare Function InitScale(x As single, y As single, z As Single) As matrix_4d
	Declare Function InitPerspective(fov As single, aspectRatio As single, zNear As single, zFar As Single) As matrix_4d
	Declare Function InitOrthographic(lefts As Single, rights As single, bottom As single, top As single, near As single, far As single) As matrix_4d

	Declare Function Transform(r As vector_4d) As vector_4d
	
	Declare Function mul(r As matrix_4d) As matrix_4d
	Declare Function toString() As string
End Type
Constructor matrix_4d()
	'm=New Single Ptr[4]
	'm[0]=New Single[4]
	'm[1]=New Single[4]
	'm[2]=New Single[4]
	'm[3]=New Single[4]
End Constructor
Destructor matrix_4d()	
	'Delete [] m[3]
	'Delete [] m[2]
	'Delete [] m[1]
	'Delete [] m[1]
	'Delete [] m
End Destructor
Function matrix_4d.InitIdentity() As matrix_4d
	m(0,0)=1:	m(0,1 )=0:	m(0,2)=0:	m(0,3)=0
	m[1][0]=0:	m[1][1]=1:	m[1][2]=0:	m[1][3]=0
	m[2][0]=0:	m[2][1]=0:	m[2][2]=1:	m[2][3]=0
	m[3][0]=0:	m[3][1]=0:	m[3][2]=0:	m[3][3]=1
	Return this	
End Function
Function matrix_4d.InitScreenSpaceTransform(halfwidth As single, halfheight As Single) As matrix_4d
	m(0,0)=halfwidth:	m(0,1 )=0:					m(0,2)=0:	m(0,3)=halfwidth-0.5
	m[1][0]=0:				m[1][1]=-halfheight:		m[1][2]=0:	m[1][3]=halfheight-0.5
	m[2][0]=0:				m[2][1]=0:					m[2][2]=1:	m[2][3]=0
	m[3][0]=0:				m[3][1]=0:					m[3][2]=0:	m[3][3]=1
	Return this	
End Function
Function matrix_4d.InitTranslation(x As single, y As single, z As Single) As matrix_4d
	m(0,0)=1:	m(0,1 )=0:	m(0,2)=0:	m(0,3)=x
	m[1][0]=0:	m[1][1]=1:	m[1][2]=0:	m[1][3]=y
	m[2][0]=0:	m[2][1]=0:	m[2][2]=1:	m[2][3]=z
	m[3][0]=0:	m[3][1]=0:	m[3][2]=0:	m[3][3]=1	
	Return This
End Function
Function matrix_4d.InitRotation(x As single, y As single, z As Single) As matrix_4d
	Dim rx As matrix_4d
	Dim ry As matrix_4d
	Dim rz As matrix_4d
	
	rz.m(0,0)=Cos(z):	rz.m(0,1 )=-Sin(z):	rz.m(0,2)=0:			rz.m(0,3)=0
	rz.m[1][0]=Sin(z):	rz.m[1][1]= Cos(z):	rz.m[1][2]=0:			rz.m[1][3]=0
	rz.m[2][0]=0:			rz.m[2][1]=0:			rz.m[2][2]=1:			rz.m[2][3]=0
	rz.m[3][0]=0:			rz.m[3][1]=0:			rz.m[3][2]=0:			rz.m[3][3]=1

	rx.m(0,0)=1:			rx.m(0,1 )=0:			rx.m(0,2)=0:			rx.m(0,3)=0
	rx.m[1][0]=0:			rx.m[1][1]=Cos(x):	rx.m[1][2]=-Sin(x):	rx.m[1][3]=0
	rx.m[2][0]=0:			rx.m[2][1]=Sin(x):	rx.m[2][2]= Cos(x):	rx.m[2][3]=0
	rx.m[3][0]=0:			rx.m[3][1]=0:			rx.m[3][2]=0:			rx.m[3][3]=1

	ry.m(0,0)=Cos(y):	ry.m(0,1 )=0:			ry.m(0,2)=-Sin(y):	ry.m(0,3)=0
	ry.m[1][0]=0:			ry.m[1][1]=1:			ry.m[1][2]=0:			ry.m[1][3]=0
	ry.m[2][0]=Sin(y):	ry.m[2][1]=0:			ry.m[2][2]= Cos(y):	ry.m[2][3]=0
	ry.m[3][0]=0:			ry.m[3][1]=0:			ry.m[3][2]=0:			ry.m[3][3]=1

	Return rz.mul(ry.mul(rx))
End Function
Function matrix_4d.InitScale(x As single, y As single, z As Single) As matrix_4d
	m(0,0)=x:	m(0,1 )=0:	m(0,2)=0:	m(0,3)=0
	m[1][0]=0:	m[1][1]=y:	m[1][2]=0:	m[1][3]=0
	m[2][0]=0:	m[2][1]=0:	m[2][2]=z:	m[2][3]=0
	m[3][0]=0:	m[3][1]=0:	m[3][2]=0:	m[3][3]=1	
	Return This
End Function
Function matrix_4d.InitPerspective(fov As single, aspectRatio As single, zNear As single, zFar As Single) As matrix_4d
	Dim As Single tanHalfFOV = tan(fov / 2.0)
	If tanHalfFOV<EPSILON Then tanHalfFOV=EPSILON 
	Dim As Single zRange = zNear - zFar
	m(0,0) = 1.0 / (tanHalfFOV * aspectRatio): m(0,1 ) = 0						: m(0,2) = 0							: m(0,3) = 0
	m[1][0] = 0											: m[1][1] = 1.0 / tanHalfFOV	: m[1][2] = 0							: m[1][3] = 0
	m[2][0] = 0											: m[2][1] = 0						: m[2][2] = (-zNear -zFar)/zRange: m[2][3] = (2 * zFar * zNear) / zRange:
	m[3][0] = 0											: m[3][1] = 0						: m[3][2] = 1							: m[3][3] = 0
Return This
End Function
Function matrix_4d.InitOrthographic(lefts As Single, rights As single, bottom As single, top As single, near As single, far As single) As matrix_4d
		Dim As Single widths = rights - lefts
		Dim As Single height = top - bottom
		Dim As single depth = far - near

		m(0,0) = 2/widths:	m(0,1 ) = 0:			m(0,2) = 0:			m(0,3) = -(rights + lefts)/widths
		m[1][0] = 0:			m[1][1] = 2/height:	m[1][2] = 0:			m[1][3] = -(top + bottom)/height
		m[2][0] = 0:			m[2][1] = 0:			m[2][2] = -2/depth:	m[2][3] = -(far + near)/depth
		m[3][0] = 0:			m[3][1] = 0:			m[3][2] = 0:			m[3][3] = 1

		return This
End Function
Function matrix_4d.Transform(r As vector_4d) As vector_4d
	Return 	vector_4d(  m(0,0) * r.GetX() + m(0,1 ) * r.GetY() + m(0,2) * r.GetZ() + m(0,3)*r.GetW(), _
								m[1][0] * r.GetX() + m[1][1] * r.GetY() + m[1][2] * r.GetZ() + m[1][3]*r.GetW(), _
								m[2][0] * r.GetX() + m[2][1] * r.GetY() + m[2][2] * r.GetZ() + m[2][3]*r.GetW(), _
								m[3][0] * r.GetX() + m[3][1] * r.GetY() + m[3][2] * r.GetZ() + m[3][3]*r.GetW())
End Function
Function matrix_4d.mul(r As matrix_4d) As matrix_4d
	Dim res As matrix_4d
	For i As UInteger=0 To 3
		For j As UInteger=0 To 3
			res.m[i][j] =  m[i][0]*r.m[0][j]+ _
								m[i][1]*r.m[1][j]+ _
								m[i][2]*r.m[2][j]+ _
								m[i][3]*r.m[3][j]
		Next
	Next
	Return res
End Function
Function matrix_4d.toString() As String
Return  " " & m(0,0) & " " & m(0,1 ) & " " & m(0,2) & " " & m(0,3) & !"\n" _ 
		& " " & m[1][0] & " " & m[1][1] & " " & m[1][2] & " " & m[1][3] & !"\n" _
		& " " & m[2][0] & " " & m[2][1] & " " & m[2][2] & " " & m[2][3] & !"\n" _
		& " " & m[3][0] & " " & m[3][1] & " " & m[3][2] & " " & m[3][3]
End Function

#EndIf