#Ifndef vector_2d_t
#Define vector_2d_t
Type vector_2d
	x as single
	y as single
	Declare Constructor ()
	Declare Constructor (x As Single,y As single)
	Declare Constructor (v As vector_2d)
	Declare Destructor ()

	Declare OPERATOR CAST() AS String
	Declare OPERATOR += (v2 As vector_2d)
	Declare OPERATOR -= (v2 As vector_2d)
	Declare OPERATOR *= (scalar As Single)
	Declare OPERATOR /= (scalar As single)

	Declare Function	Clone() As vector_2d
	Declare Function 	GetLenght() As Single
	Declare Function  GetDistance(dest As vector_2d) As Single
	Declare Function 	GetAngle() As Single
	Declare Function	GetAngleDeg() As Single
	Declare Function 	GetDirectionVector(dest As vector_2d) As vector_2d
	Declare Function 	GetDirectionAngle(dest As vector_2d) As Single
	Declare Function 	GetDotProduct(v As vector_2d) As Single
	Declare Function 	GetCrossProduct(v As vector_2d) As Single
	Declare Function 	GetInterpolate( A As vector_2d, B As vector_2d, t As single ) As vector_2d
	Declare Function	TriangleAreaTimesTwo(b As vector_2d,c As vector_2d) As single
	Declare Function 	toString() As String
	'Declare Function	Add(scalar As single) As single
	'Declare Function	Sub(scalar As single) As single
	Declare Function	Mul(scalar As single) As single
	
	Declare Sub 		SetAngle(angle As single)
	Declare Sub 		SetAngleDeg(angle As single)
	Declare Sub			Normalize()
	Declare Sub 		Rotate(angle As single)
	Declare Sub 		Turn(angle As single)
	Declare Sub 		RotateAt(pivot As vector_2d, angle As single)
	Declare Sub 		TurnAt(v As vector_2d, angle As single)
	Declare Sub			Position(v As vector_2d)
	Declare Sub			Translate(v As vector_2d)
	Declare Sub			Move(v As vector_2d)
	Declare Sub 		LookAt(v As vector_2d)
	Declare Sub 		Reflect(v As vector_2d)
	Declare Sub			Invert()
	Declare Sub 		Limit(max As single)
End Type
OPERATOR = (v1 AS vector_2d, v2 AS vector_2d) AS vector_2d
	RETURN vector_2d( v1.x = v2.x, v1.y = v2.y )
END Operator
Operator + (v1 AS vector_2d, v2 AS vector_2d) AS vector_2d
	RETURN vector_2d( v1.x + v2.x, v1.y + v2.y )
END Operator
OPERATOR vector_2d.+= (v2 AS vector_2d)
	this.x=this.x+v2.x
	this.y=this.y+v2.y
End Operator
Operator - (v1 AS vector_2d, v2 AS vector_2d) AS vector_2d
	RETURN vector_2d( v1.x - v2.x, v1.y - v2.y )
END Operator
OPERATOR vector_2d.-= (v2 AS vector_2d)
	this.x=this.x-v2.x
	this.y=this.y-v2.y
End Operator
Operator * (v1 As vector_2d, scalar As Single) AS vector_2d
	RETURN vector_2d( v1.x * scalar, v1.y * scalar )
END Operator


OPERATOR vector_2d.*= (scalar AS Single)
	this.x*=scalar
	this.y*=scalar
End Operator
Operator / (v1 As vector_2d, scalar As Single) AS vector_2d
	RETURN vector_2d( v1.x / scalar, v1.y / scalar )
END Operator
OPERATOR vector_2d./= (scalar As Single)
	If scalar=0 Then scalar=1
	this.x=this.x/scalar
	this.y=this.y/scalar
End Operator

Constructor vector_2d()
this.x=0
this.y=0
End Constructor
Constructor vector_2d(x As Single,y As Single)
this.x=x
this.y=y
End Constructor
Constructor vector_2d(v As vector_2d)
This.x=v.x
this.y=v.y
End Constructor
OPERATOR vector_2d.CAST () AS STRING
	RETURN "(" + STR(this.x) + ", " + STR(this.y) + ")"
END OPERATOR
Destructor vector_2d()

End Destructor
'Function	Mul(scalar As single) As Single
'	RETURN vector_2d( v1.x * scalar, v1.y * scalar )	
'End Function
Function vector_2d.Clone() As vector_2d
	Return vector_2d(this)
End Function
Function vector_2d.GetLenght() As Single
	return sqr(this.x*this.x + this.y*this.y)
End Function
Function vector_2d.GetAngle() As Single
	Return ATan2( x, y )
End Function
Function vector_2d.GetAngleDeg() As Single
	Return ATan2( x, y )*180/M_PI
End Function

Function vector_2d.GetDistance(dest As vector_2d) As Single
	Dim delta As vector_2d=dest-this
	Dim dist As Single=sqr(delta.x*delta.x+delta.y*delta.y)
	Return dist
End Function
Function vector_2d.GetDirectionVector(dest As vector_2d) As vector_2d
	Dim delta As vector_2d=dest-This
	Dim rad As Single=ATan2(delta.x,delta.y)
	if rad<0 then rad=M_PI2+rad
	delta.x=Sin(rad)
	delta.y=Cos(rad)
	Return delta
End Function
Function vector_2d.GetDirectionAngle(dest As vector_2d) As Single
	Dim delta As vector_2d=dest-This
	Dim rad As Single=ATan2(delta.x,delta.y)
	Return rad
End Function
Function vector_2d.GetDotProduct(v As vector_2d) As Single
	return (this.x * v.x) + (this.y * v.y)
End Function
Function vector_2d.GetCrossProduct(v As vector_2d) As Single
	return (this.x * v.y) - (this.y * v.x)
End Function
Function vector_2d.GetInterpolate( A As vector_2d, B As vector_2d, t As single ) As vector_2d
	return A*t + B*(1.0-t)
End Function

sub vector_2d.SetAngle(angle As Single)
	Dim As single length = Sqr(x*x+y*y)
	this.x = cos(angle)*length
	this.y = sin(angle)*length
End Sub
sub vector_2d.SetAngleDeg(angle  As Single)
	Dim As single length = Sqr(x*x+y*y)
	angle *= (M_PI/180)
	this.x = cos(angle)*length
	this.y = sin(angle)*length
End Sub
Sub vector_2d.Normalize()
	Dim mag As Single=Sqr(this.x*this.x+this.y*this.y)
	If mag<EPSILON Then
		This*=0
	Else
		dim invLength As single= 1.0 / mag
		This *= invLength
	endif
End Sub
Sub vector_2d.Rotate(angle As single)
	Dim As Single xt = (this.x * cos(angle)) - (this.y * Sin(angle))
	Dim As single yt = (this.y * Cos(angle)) + (this.x * sin(angle))
	this.x = xt
	this.y = yt
End Sub
Sub vector_2d.RotateAt(pivot As vector_2d,angle As single)
	Dim As Single xt = pivot.x+Cos(angle)*(this.x-pivot.x) - Sin(angle)*(this.y-pivot.y)
	Dim As single yt = pivot.y+Sin(angle)*(this.x-pivot.x) + Cos(angle)*(this.y-pivot.y)
	this.x = xt
	this.y = yt
End Sub
Sub vector_2d.Turn(angle As single)
	Dim As single l = this.getAngle()
	Dim As single length = sqr(x*x+y*y)
	this.x = cos(angle+l)*length
	this.y = sin(angle+l)*length
End Sub
Sub vector_2d.TurnAt(v As vector_2d, angle As single)
	Dim As Single xt = v.x+Cos(angle)*(this.x-v.x) - Sin(angle)*(this.y-v.y)
	Dim As single yt = v.y+sin(angle)*(this.x-v.x) + Cos(angle)*(this.y-v.y)
	this.x = xt
	this.y = yt
End Sub
Sub vector_2d.LookAt(v As vector_2d)
	dim vectorToTarget As vector_2d= vector_2d(v.x - this.x, v.y - this.y )
	this.setAngle( vectorToTarget.getAngle())
End Sub
Sub vector_2d.Reflect(v As vector_2d)
	Dim unitNormal As vector_2d=v.Clone()
	unitNormal.Normalize()
	this = this - (unitNormal*((unitNormal.GetDotProduct(this)) * 2 ))
End Sub
Sub vector_2d.Position(v As vector_2d)
	this=v
End Sub
Sub vector_2d.Translate(v As vector_2d)
	This+=v
End Sub
Sub vector_2d.Move(v As vector_2d)
	This+=v
End Sub
Sub vector_2d.Invert()
	this.x=-this.x
	this.y=-this.y
End Sub
Sub vector_2d.Limit(max As single)
	If (this.GetLenght() > max) Then
		this.Normalize()
		This*=max
	endif
End Sub
Function	vector_2d.TriangleAreaTimesTwo(b As vector_2d,c As vector_2d) As Single
	Dim x1 As Single = b.x-x
	Dim y1 As Single = b.y-y
	Dim x2 As Single = c.x-x
	Dim y2 As Single = c.y-y
	
	Return (x1*y2-x2*y1)
End Function
Function vector_2d.toString() As String
Return  " " & x & " " & y
End Function
#endif