#Ifndef vector_3d_t
#Define vector_3d_t
Type vector_3d
	x as single
	y as Single
	z As Single
	
	Declare Constructor ()
	Declare Constructor (x As Single,y As Single,z As Single)
	Declare Constructor (v As vector_3d)
	Declare Destructor ()

	Declare OPERATOR CAST() AS String
	Declare OPERATOR += (v2 As vector_3d)
	Declare OPERATOR -= (v2 As vector_3d)
	Declare OPERATOR *= (scalar As Single)
	Declare OPERATOR /= (scalar As single)

	Declare Function	Clone() As vector_3d
	Declare Function 	GetLenght() As Single
	Declare Function  GetDistance(dest As vector_3d) As Single
	Declare Function 	GetAngle() As Single
	Declare Function	GetAngleDeg() As Single
	Declare Function 	GetDirectionVector(dest As vector_3d) As vector_3d
	Declare Function 	GetDirectionAngle(dest As vector_3d) As Single
	Declare Function 	GetDotProduct(v As vector_3d) As Single
	Declare Function 	GetCrossProduct(v As vector_3d) As vector_3d
	Declare Function 	GetInterpolate( A As vector_3d, B As vector_3d, t As single ) As vector_3d
	Declare Function	TriangleAreaTimesTwo(b As vector_3d,c As vector_3d) As single
	Declare Function 	toString() As String

	Declare Sub 		SetAngle(angle As single)
	Declare Sub 		SetAngleDeg(angle As single)
	Declare Sub			Normalize()
	Declare Sub 		Rotate(angle As single)
	Declare Sub 		Turn(angle As single)
	Declare Sub 		RotateAt(pivot As vector_3d, angle As single)
	Declare Sub 		TurnAt(v As vector_3d, angle As single)
	Declare Sub			Position(v As vector_3d)
	Declare Sub			Translate(v As vector_3d)
	Declare Sub			Move(v As vector_3d)
	Declare Sub 		LookAt(v As vector_3d)
	Declare Sub 		Reflect(v As vector_3d)
	Declare Sub			Invert()
	Declare Sub 		Limit(max As single)
End Type
OPERATOR = (v1 AS vector_3d, v2 AS vector_3d) AS vector_3d
	RETURN vector_3d( v1.x = v2.x, v1.y = v2.y, v1.z = v2.z )
END Operator
Operator + (v1 AS vector_3d, v2 AS vector_3d) AS vector_3d
	RETURN vector_3d( v1.x + v2.x, v1.y + v2.y,v1.z + v2.z )
END Operator
OPERATOR vector_3d.+= (v2 AS vector_3d)
	this.x=this.x+v2.x
	this.y=this.y+v2.y
	this.z=this.z+v2.z
End Operator
Operator - (v1 AS vector_3d, v2 AS vector_3d) AS vector_3d
	RETURN vector_3d( v1.x - v2.x, v1.y - v2.y, v1.z - v2.z)
END Operator
OPERATOR vector_3d.-= (v2 AS vector_3d)
	this.x=this.x-v2.x
	this.y=this.y-v2.y
	this.z=this.z-v2.z
End Operator
Operator * (v1 As vector_3d, scalar As Single) AS vector_3d
	RETURN vector_3d( v1.x * scalar, v1.y * scalar, v1.z * scalar )
END Operator
OPERATOR vector_3d.*= (scalar AS Single)
	this.x*=scalar
	this.y*=scalar
	this.z*=scalar
End Operator
Operator / (v1 As vector_3d, scalar As Single) AS vector_3d
	RETURN vector_3d( v1.x / scalar, v1.y / scalar, v1.z / scalar )
END Operator
OPERATOR vector_3d./= (scalar As Single)
	If scalar=0 Then scalar=1
	this.x=this.x/scalar
	this.y=this.y/scalar
	this.z=this.z/scalar
End Operator

Constructor vector_3d()
this.x=0
this.y=0
this.z=0
End Constructor
Constructor vector_3d(x As Single,y As Single, z As Single)
this.x=x
this.y=y
this.z=z
End Constructor
Constructor vector_3d(v As vector_3d)
This.x=v.x
this.y=v.y
this.z=v.z
End Constructor
OPERATOR vector_3d.CAST () AS STRING
	RETURN "(" + STR(this.x) + ", " + STR(this.y) + ", " + STR(this.z) + ")"
END OPERATOR
Destructor vector_3d()

End Destructor

Function vector_3d.Clone() As vector_3d
	Return vector_3d(this)
End Function
Function vector_3d.GetLenght() As Single
	return sqr(this.x*this.x + this.y*this.y + this.z*this.z)
End Function
Function vector_3d.GetAngle() As Single
	Return ATan2( x, y )
End Function
Function vector_3d.GetAngleDeg() As Single
	Return ATan2( x, y )*180/M_PI
End Function

Function vector_3d.GetDistance(dest As vector_3d) As Single
	Dim delta As vector_3d=dest-this
	Dim dist As Single=sqr(delta.x*delta.x+delta.y*delta.y+delta.z*delta.z)
	Return dist
End Function
Function vector_3d.GetDirectionVector(dest As vector_3d) As vector_3d
	Dim delta As vector_3d=dest-This
	Dim rad As Single=ATan2(delta.x,delta.y)
	if rad<0 then rad=M_PI2+rad
	delta.x=Sin(rad)
	delta.y=Cos(rad)
	Return delta
End Function
Function vector_3d.GetDirectionAngle(dest As vector_3d) As Single
	Dim delta As vector_3d=dest-This
	Dim rad As Single=ATan2(delta.x,delta.y)
	Return rad
End Function
Function vector_3d.GetDotProduct(v As vector_3d) As Single
	return (this.x * v.x) + (this.y * v.y) + (this.z * v.z)
End Function
Function vector_3d.GetCrossProduct(v As vector_3d) As vector_3d	
	Return vector_3d(y*v.z - z*v.y, z*v.x - x*v.z, x*v.y - y*v.x)
End Function
Function vector_3d.GetInterpolate( A As vector_3d, B As vector_3d, t As single ) As vector_3d
	return A*t + B*(1.0-t)
End Function

sub vector_3d.SetAngle(angle As Single)
	Dim As single length = Sqr(x*x+y*y)
	this.x = cos(angle)*length
	this.y = sin(angle)*length
End Sub
sub vector_3d.SetAngleDeg(angle  As Single)
	Dim As single length = Sqr(x*x+y*y)
	angle *= (M_PI/180)
	this.x = cos(angle)*length
	this.y = sin(angle)*length
End Sub
Sub vector_3d.Normalize()
	Dim mag As Single=Sqr(this.x*this.x+this.y*this.y+this.z*this.z)
	If mag<EPSILON Then
		This*=0
	Else
		dim invLength As single= 1.0 / mag
		This *= invLength
	endif
End Sub
Sub vector_3d.Rotate(angle As single)
	Dim As Single xt = (this.x * cos(angle)) - (this.y * Sin(angle))
	Dim As single yt = (this.y * Cos(angle)) + (this.x * sin(angle))
	this.x = xt
	this.y = yt
End Sub
Sub vector_3d.RotateAt(pivot As vector_3d,angle As single)
	Dim As Single xt = pivot.x+Cos(angle)*(this.x-pivot.x) - Sin(angle)*(this.y-pivot.y)
	Dim As single yt = pivot.y+Sin(angle)*(this.x-pivot.x) + Cos(angle)*(this.y-pivot.y)
	this.x = xt
	this.y = yt
End Sub
Sub vector_3d.Turn(angle As single)
	Dim As single l = this.getAngle()
	Dim As single length = sqr(x*x+y*y)
	this.x = cos(angle+l)*length
	this.y = sin(angle+l)*length
End Sub
Sub vector_3d.TurnAt(v As vector_3d, angle As single)
	Dim As Single xt = v.x+Cos(angle)*(this.x-v.x) - Sin(angle)*(this.y-v.y)
	Dim As single yt = v.y+sin(angle)*(this.x-v.x) + Cos(angle)*(this.y-v.y)
	this.x = xt
	this.y = yt
End Sub
Sub vector_3d.LookAt(v As vector_3d)
	dim vectorToTarget As vector_3d= vector_3d(v.x - this.x, v.y - this.y , v.z - this.z )
	this.setAngle( vectorToTarget.getAngle())
End Sub
Sub vector_3d.Reflect(v As vector_3d)
	Dim unitNormal As vector_3d=v.Clone()
	unitNormal.Normalize()
	this = this - (unitNormal*((unitNormal.GetDotProduct(this)) * 2 ))
End Sub
Sub vector_3d.Position(v As vector_3d)
	this=v
End Sub
Sub vector_3d.Translate(v As vector_3d)
	This+=v
End Sub
Sub vector_3d.Move(v As vector_3d)
	This+=v
End Sub
Sub vector_3d.Invert()
	this.x=-this.x
	this.y=-this.y
	this.z=-this.z
End Sub
Sub vector_3d.Limit(max As single)
	If (this.GetLenght() > max) Then
		this.Normalize()
		This*=max
	endif
End Sub
Function	vector_3d.TriangleAreaTimesTwo(b As vector_3d,c As vector_3d) As Single
	Dim x1 As Single = b.x-x
	Dim y1 As Single = b.y-y
	Dim x2 As Single = c.x-x
	Dim y2 As Single = c.y-y
	
	Return (x1*y2-x2*y1)
End Function
Function vector_3d.toString() As String
Return  " " & x & " " & y & " " & z
End Function
#endif